function [S L objlist timelist] = mysolver(Sigma, alpha, beta, maxit)

maxpgiter = 50; %number of PG iterations for solving the nuclear norm subproblem. 
maxcditer = 5; % number of CD iterations for solving the l1 regularized subproblem. 
maxlineiter = 20; 
maxinneriter = 3;
%% initial
p = size(Sigma,1);
L = zeros(p,p);
U = [];
S = eye(p,p);
fprintf('initial obj: %g\n', CompObj(Sigma, S, L, alpha, beta));
timebegin = cputime;
stepsize = 1;

for iter = 1:maxit
	
	Y = inv(S-L);
	newL = L;
	newS = S;
%% active subspace for nuclear norm (active variables will be selected in QUIC)
	grad = Y-Sigma;
	[uu ss] = mexeig(grad);
	gind = find(ss < -beta*1.1);
	UG = uu(:, gind); 
	UA = orth([UG U]);
	k = size(UA,2);

	D = UA'*L*UA;
	Ybar = UA'*Y*UA;
	beginGbar = UA'*grad*UA;
	newD = D;
	invYbar =inv(Ybar);

	for inneriter = 1:maxinneriter
		%% Fix L solve S
		AA = Sigma + Y*(newS-S-newL+L)*Y;
		[X] = QUICquad(AA, alpha, 0, maxcditer, newS, Y); 
		newS = X;

		%% Fix S solve L
		stepsize = 1; % PG step size
		nowGbar = beginGbar + (UA'*Y*(S-newS))*(Y*UA);

		%% test the close form solution
		deltaD = invYbar*(nowGbar+beta*eye(k))*invYbar;
		newnewD = newD-deltaD;
		[u s] = mexeig(newnewD);
		if min(s) >-1e-8
%			fprintf('close form successful\n');
			newD = newnewD;
		else
%			fprintf('close form unsuccessful\n');
			%% projected gradient for solving the subproblem
			for i=1:maxpgiter
				grad = nowGbar +  Ybar*(newD-D)*Ybar + beta*eye(k);
				nowobj = CompQuadObj_UA(Ybar, nowGbar, newD, D, beta);
				for j=stepsize:maxlineiter+stepsize
					newnewD = newD - (2^(1-j))*grad;
					[u s] = mexeig(newnewD);
					newnewD = u*diag(max(s,0))*u';
					newobj = CompQuadObj_UA(Ybar, nowGbar, newnewD, D, beta);
					if (newobj < nowobj)
						stepsize = max(j-1,1);
						break;
					end	
				end
				if (j==maxlineiter+stepsize )
%					fprintf('WARNING: line search fail\n');
					break
				end
				nowobj = newobj;
				newD = newnewD;
				newD = (newD+newD')/2;
			end
		end

		newL = UA*newD*UA';
		newL = (newL+newL')/2;
%		fprintf('inneriter %g After Updating L\n', inneriter);
	end
	newL = UA*newD*UA';
	newL = (newL+newL')/2;

	deltaS = newS-S;
	deltaL = newL - L;
	if (iter==1)
		nowobj_real = CompObj(Sigma, S, L, alpha, beta);
	end
	%% line search
	for j = 1:maxlineiter
		s = 2^(-j+1);
		SS = S + s*deltaS;
		LL = L + s*deltaL;
		newobj_real = CompObj(Sigma, SS, LL, alpha, beta);
		if (newobj_real < nowobj_real)
			S = SS;
			L = LL;
%			fprintf('Step size: %g, oldobj: %g, newobj %g time %g\n', s, nowobj_real, newobj_real, cputime-timebegin);
			break;
		end
	end
	if j==maxlineiter
		fprintf('Newton Line Search Fail\n');
	end
	nowobj_real = newobj_real;
	[qq rr] = qr(L,0);
	ind = find(abs(diag(rr))>1e-8);
	U = qq(:,ind);

	objlist(iter) = newobj_real;
	timelist(iter) = cputime - timebegin;

	fprintf('Iter %g time %g obj %g nonzeros %g rank %g\n', iter, cputime - timebegin, newobj_real, nnz(SS), numel(ind));
end
